package es.prodevelop.tilecache.draw;

import es.prodevelop.gvsig.mini.common.IBitmap;
import es.prodevelop.gvsig.mini.common.ICanvas;
import es.prodevelop.gvsig.mini.common.impl.Tile;

/**
 * A Class to draw tiles (IBitmaps) into a buffer canvas (ICanvas). It can be
 * used to draw to the buffer canvas on the same thread as the tiles are being
 * requested
 * 
 * @author aromeu
 * 
 */
public class TileDrawer {

	ICanvas bufferCanvas;
	Object paint;

	/**
	 * 
	 * @param bufferCanvas
	 * @param paint
	 */
	public TileDrawer(ICanvas bufferCanvas, Object paint) {
		this.bufferCanvas = bufferCanvas;
	}

	/**
	 * Draws a bitmap that represents a tile into the buffer canvas
	 * 
	 * @param t
	 *            The tile to draw
	 * @param bitmap
	 *            The bitmap to draw
	 */
	public void drawTile(Tile t, IBitmap bitmap) {
		if (!t.cancellable.getCanceled())
			this.bufferCanvas.drawBitmap(bitmap, t.distanceFromCenter.getX(),
					t.distanceFromCenter.getY(), paint);
	}

	/**
	 * @return the bufferCanvas
	 */
	public ICanvas getBufferCanvas() {
		return bufferCanvas;
	} 

	/**
	 * @param bufferCanvas
	 *            the bufferCanvas to set
	 */
	public void setBufferCanvas(ICanvas bufferCanvas) {
		this.bufferCanvas = bufferCanvas;
	}

	/**
	 * @return the paint
	 */
	public Object getPaint() {
		return paint;
	}

	/**
	 * @param paint
	 *            the paint to set
	 */
	public void setPaint(Object paint) {
		this.paint = paint;
	}

}
