/* TileRasterCache, a library to download tiles from several sources.
 *
 * Copyright (C) 2010 Prodevelop SL.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *  
 *   Prodevelop Integracin de Tecnologas SL
 *   Plaza don Juan de Vilarrasa , 14-5
 *   46000 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *   
 *   author: Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */

package es.prodevelop.tilecache.generator;

/**
 * This interface is used to provide a buffer around the tiles to be downloaded.
 * Some strategies can be applied: A buffer that fits the size of the screen (,
 * a buffer that provides one more tile around the screen, a buffer that
 * provides the double of the tiles needed around the screen, etc.
 * 
 * @author aromeu
 * 
 */
public interface IBufferStrategy {

	/**
	 * Calculates the buffer. The buffer is expressed as the number of tiles
	 * around tile that is at the center of the screen
	 * 
	 * @return res[0] => additionalTilesNeededToLeftOfCenter; res[1] =>
	 *         additionalTilesNeededToRightOfCenter; res[2] =>
	 *         additionalTilesNeededToTopOfCenter; res[3] =>
	 *         additionalTilesNeededToBottomOfCenter;
	 */
	public int[] calculateBufferAroundCenterTile(int mapWidthPx,
			int mapHeightPx, int tileSizePx, int centerMapTileScreenLeft,
			int centerMapTileScreenRight, int centerMapTileScreenTop,
			int centerMapTileScreenBottom);

	/**
	 * The max tile number given a zoomLevel
	 * 
	 * @return
	 */
	public int calculateTileUpperBound(int zoomLevel);

}
