/* TileRasterCache, a library to download tiles from several sources.
 *
 * Copyright (C) 2010 Prodevelop SL.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *  
 *   Prodevelop Integraci�n de Tecnolog�as SL
 *   Plaza don Juan de Vilarrasa , 14-5
 *   46000 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *   
 *   author: Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */

package es.prodevelop.tilecache.generator;

import es.prodevelop.gvsig.mini.common.impl.Tile;

/**
 * Implementations of this interface should provide a method to order a set of
 * tiles. For example, a set of tiles can be ordered by distance to the center
 * of the screen, from top to bottom and left to right, by area shown on the
 * screen, etc.
 * 
 * @author aromeu
 * 
 */
public interface ITileSorter {
	
	/**
	 * /**
	 * Sorts an array of tiles.
	 * @param tiles The sorted tiles
	 * @param centerPixelX The X pixel of the center of the map
	 * @param centerPixelY The Y pixel of the center of the map
	 * @param tileSizePx The size of the tile in pixels (Usually 256)
	 */
	public void sortTiles(Tile[] tiles, int centerPixelX, int centerPixelY, int tileSizePx);

}
