/* TileRasterCache, a library to download tiles from several sources.
 *
 * Copyright (C) 2010 Prodevelop SL.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *  
 *   Prodevelop Integraci�n de Tecnolog�as SL
 *   Plaza don Juan de Vilarrasa , 14-5
 *   46000 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *   
 *   author: Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */

package es.prodevelop.tilecache.generator.impl;

import es.prodevelop.gvsig.mini.geom.Pixel;
import es.prodevelop.tilecache.generator.ITileSorter;
import es.prodevelop.gvsig.mini.common.impl.CollectionQuickSort;
import es.prodevelop.gvsig.mini.common.impl.Tile;

public class DistanceFromCenterTileSorter implements ITileSorter {

	/**
	 * Sorts the tiles from the center of the screen to outside to load the
	 * tiles in spiral.
	 * 
	 * @param array2
	 *            the array of tiles
	 * @param centerPixelX The X pixel of the center of the map
	 * @param centerPixelY The Y pixel of the center of the map
	 * @param tileSizePx The size of the tile in pixels (Usually 256)
	 */
	public void sortTiles(final Tile[] array2, final int centerPixelX, final int centerPixelY, final int tileSizePx) {
		try {
			final Pixel center = new Pixel(centerPixelX, centerPixelY);
			double e1, e2;
			final int length = array2.length;
			Tile t;
			Tile t1;
			final Pixel temp = new Pixel((tileSizePx / 2), (tileSizePx / 2));
			for (int pass = 1; pass < length; pass++) {
				for (int element = 0; element < length - 1; element++) {
					t = array2[element];
					t1 = array2[element + 1];
					if (t != null && t1 != null) {
						e1 = (t.distanceFromCenter.add(temp)).distance(center);
						e2 = (t1.distanceFromCenter.add(temp)).distance(center);
						if (e1 >= e2) {
							swap(array2, element, element + 1);
						}
					}
				}
			}
		} catch (final Exception e) {
			
		}
	}

	private void swap(final Tile[] array3, final int first, final int second) {
		final Tile hold = array3[first];
		array3[first] = array3[second];
		array3[second] = hold;
	}
}
