/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2009 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2009.
 *   author Rub�n Blanco rblanco@prodevelop.es
 *
 *
 * Original version of the code made by Nicolas Gramlich.
 * No header license code found on the original source file.
 * 
 * Original source code downloaded from http://code.google.com/p/osmdroid/
 * package org.andnav.osm.views.util;
 * 
 * License stated in that website is 
 * http://www.gnu.org/licenses/lgpl.html
 * As no specific LGPL version was specified, LGPL license version is LGPL v2.1
 * is assumed.
 *  
 * 
 */

package es.prodevelop.tilecache.provider;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import es.prodevelop.gvsig.mini.common.IBitmap;
import es.prodevelop.gvsig.mini.common.IContext;
import es.prodevelop.gvsig.mini.common.IHandler;
import es.prodevelop.gvsig.mini.common.impl.BaseEvent;
import es.prodevelop.gvsig.mini.common.impl.Tile;
import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.map.GeoUtils;
import es.prodevelop.gvsig.mini.utiles.Cancellable;
import es.prodevelop.gvsig.mini.utiles.Constants;
import es.prodevelop.gvsig.mini.utiles.WorkQueue;
import es.prodevelop.tilecache.draw.TileDrawer;
import es.prodevelop.tilecache.provider.event.TileEvent;
import es.prodevelop.tilecache.provider.filesystem.ITileFileSystemProvider;

/**
 * Class which download tiles launching a background task on the WorkQueue and
 * saves the Bitmap generated on the TileFilesystemProvider
 * 
 * @author aromeu
 * @author rblanco
 * 
 */
public class Downloader implements GeoUtils {

	private final static Logger log = Logger.getLogger("Downloader");
	protected TileCache mTileCache;
	protected HashSet<String> mPending = new HashSet<String>();
	protected IContext mCtx;
	public int transfer = 0;
	String datalog = null;
	protected ITileFileSystemProvider mMapTileFSProvider;
	private Extent viewExtent;

	// protected ExecutorService mThreadPool = Executors.newFixedThreadPool(3);

	/**
	 * The constructor
	 */
	public Downloader(final IContext ctx,
			final ITileFileSystemProvider aMapTileFSProvider) {
		try {
			this.mCtx = ctx;
			this.mMapTileFSProvider = aMapTileFSProvider;
			// try {
			// CompatManager.getInstance().getRegisteredLogHandler().
			// configureLogger(log);
			// } catch (BaseException e) {
			//
			// }
		} catch (Exception e) {
			log.log(Level.SEVERE, "Downloader constructor: ", e);
		}
	}

	/**
	 * Executes a background task in the Workqueue which downloads a resource
	 * from a tile server, saves the file to a TileFilesystemProvider and sends
	 * a callback to a Handler
	 * 
	 * @param aTile
	 *            The tile
	 * @param callback
	 *            The Handler to callback
	 * @param cancellable
	 *            A Cancellable instance
	 * @param drawer
	 *            A TileDrawer with a bufferCanvas to draw the tiles also in
	 *            background
	 * @param prepend
	 *            Puts the task at the beginning of the WorkQueue
	 */
	public synchronized void getRemoteImageAsync(final Tile aTile,
			final IHandler callback, final String cacheURL,
			final Cancellable cancellable, final TileDrawer drawer,
			final boolean prepend) {
		try {
			WorkQueue.getInstance().execute(new Runnable() {
				// @Override
				public void run() {
					InputStream in = null;
					OutputStream out = null;

					boolean succeed = false;
					int retryCount = 0;

					while (retryCount <= TileProvider.NUM_RETRY && !succeed) {
						try {
							// if (checkDiscard(aTile)) {
							// System.out.println("Discarded");
							// mPending.remove(cacheURL);
							// return;
							// }

							if (cancellable.getCanceled()) {
								mPending.clear();
								WorkQueue.getInstance().clearPendingTasks();
								mCtx.sendMessage(
										callback,
										Downloader.this.instantiateEvent(aTile),
										GeoUtils.TILEPROVIDER_CANCELED);
								return;
							}
							in = new BufferedInputStream(
									openConnection(aTile.mURL),
									Constants.IO_BUFFER_SIZE);
							final ByteArrayOutputStream dataStream = new ByteArrayOutputStream();
							out = new BufferedOutputStream(dataStream,
									Constants.IO_BUFFER_SIZE);
							Constants.copy(in, out);
							out.flush();
							final byte[] data = dataStream.toByteArray();
							// log.log(Level.FINE,
							// "downloaded data from server");
							final IBitmap bmp = mCtx.decodeByteArray(data, 0,
									data.length);
							if (Downloader.this.mMapTileFSProvider
									.getTileCache() != null)
								Downloader.this.mMapTileFSProvider
										.getTileCache().putTile(cacheURL, bmp);
							// log.log(Level.FINE,
							// "Bitmap put into memory cache");

							if (mCtx.isSDMounted()) {
								// log.log(Level.FINE,
								// "trying to save file to disk " + aURLString);
								Downloader.this.mMapTileFSProvider.saveFile(
										data, aTile);
							}

							// mPending.remove(aURLString);
							// mPending.add(aURLString);
							transfer = data.length + transfer;
							int transfer2 = transfer / 1024;
							mCtx.sendMessage(callback,
									new BaseEvent(String.valueOf(data.length)),
									MAPTILEDOWNLOADER_SUCCESS_SIZE);
							mCtx.sendMessage(callback,
									Downloader.this.instantiateEvent(aTile),
									MAPTILEDOWNLOADER_SUCCESS_ID);

							// log.log(Level.FINE, "Loaded from server: ");
							// mMapTileFSProvider.datas = null;
							succeed = true;
							if (drawer != null) {
								drawer.drawTile(aTile.clone(), bmp);
							}
							mPending.remove(cacheURL);
						} catch (OutOfMemoryError oe) {
							System.gc();
							log.log(Level.SEVERE, aTile.mURL);
							log.log(Level.SEVERE, "OutOfMemory: ", oe);
							Downloader.this.mMapTileFSProvider.getTileCache()
									.onLowMemory();
							if (retryCount == TileProvider.NUM_RETRY)
								mCtx.sendMessage(
										callback,
										Downloader.this.instantiateEvent(aTile),
										MAPTILEDOWNLOADER_OOM_ID);
						} catch (Exception e) {
							// TODO catch SocketExceptions
							// "The connection was reset"
							if (retryCount == TileProvider.NUM_RETRY)
								mCtx.sendMessage(
										callback,
										Downloader.this.instantiateEvent(aTile),
										MAPTILEDOWNLOADER_FAIL_ID);

							log.log(Level.SEVERE, "Tile Loading Error "
									+ aTile.mURL);
							log.log(Level.SEVERE, e.getMessage());
						} finally {
							Constants.closeStream(in);
							Constants.closeStream(out);
							retryCount++;
						}
					}

				}
			}, prepend);
		} catch (Exception e) {
			log.log(Level.SEVERE, "getRemoteImageAsync: ", e);
		}
	}

	/**
	 * Check if the resource to request is in the pending queue and if not,
	 * calls the background task to download it.
	 * 
	 * @param tile
	 *            The tile
	 * @param callback
	 *            THe handler to callback
	 * @param cancellable
	 *            a Cancellable instance
	 * @param drawer
	 *            A TileDrawer with a bufferCanvas to draw the tiles also in
	 *            background
	 * @param prepend
	 *            Puts the task at the beginning of the WorkQueue
	 */
	public synchronized void requestMapTileAsync(final Tile aTile,
			final IHandler callback, final String cacheURL,
			final Cancellable cancellable, final TileDrawer drawer,
			final boolean prepend) {
		try {
			// long t1 = System.currentTimeMillis();
			if (this.mPending.contains(cacheURL)) {
				// long t2 = System.currentTimeMillis();
				// System.out.println("Time to queue: " + (t2 - t1));
				return;
			}
			this.mPending.add(cacheURL);
			// long t2 = System.currentTimeMillis();
			// System.out.println("Time to queue: " + (t2 - t1));
			getRemoteImageAsync(aTile, callback, cacheURL, cancellable, drawer,
					prepend);
		} catch (Exception e) {
			log.log(Level.SEVERE, "requestMapTileAsync: ", e);
		}
	}

	public TileEvent instantiateEvent(Tile aTile) {
		return new TileEvent(aTile.clone(), "",
				Downloader.this.mMapTileFSProvider.getMode(),
				Downloader.this.mMapTileFSProvider.getStrategy().getName(),
				TileEvent.TILE_LEVEL_SERVER);
	}

	public void setViewExtent(Extent viewExtent) {
		this.viewExtent = viewExtent;
	}

	public Extent getViewExtent() {
		return this.viewExtent;
	}

	/**
	 * Checks if a tile intersects with the viewExtent. If not it should be
	 * discarded (not downloaded)
	 * 
	 * @param tile
	 *            The tile
	 * @return True if not intersects and should be discarded
	 */
	public boolean checkDiscard(Tile tile) {
		try {
			if (tile.extent == null || getViewExtent() == null)
				return false;

			return !getViewExtent().intersect(tile.extent);
		} catch (Exception e) {
			return false;
		}
	}

	public void removeFromPending(String cacheURL) {
		this.mPending.remove(cacheURL);

	}

	public Set getPendingQueue() {
		return mPending;
	}

	public static InputStream openConnection(String query) throws IOException {
		final URL url = new URL(query.replace(" ", "%20"));
		URLConnection urlconnec = url.openConnection();
		urlconnec.setConnectTimeout(15000);
		urlconnec.setReadTimeout(15000);		
		return urlconnec.getInputStream();
	}
}
