package es.prodevelop.tilecache.provider.filesystem;

import java.io.IOException;
import java.util.Set;

import es.prodevelop.gvsig.mini.common.IHandler;
import es.prodevelop.gvsig.mini.common.impl.Tile;
import es.prodevelop.gvsig.mini.utiles.Cancellable;
import es.prodevelop.tilecache.draw.TileDrawer;
import es.prodevelop.tilecache.provider.TileCache;
import es.prodevelop.tilecache.provider.filesystem.strategy.ITileFileSystemStrategy;

public interface ITileFileSystemProvider {

	/**
	 * Checks if the tile to download is already on the pending queue and if not
	 * launches a background task to find it on the file system and load it to
	 * the TileCache
	 * 
	 * @param tile
	 *            The tile
	 * @param callback
	 *            The Handler callback
	 * @param cancellable
	 *            A Cancellable instance
	 * @param drawer
	 *            A TileDrawer with a bufferCanvas to draw the tiles also in
	 *            background
	 * @param prepend
	 * 			  Puts the task at the beginning of the WorkQueue
	 * @throws IOException
	 * @returns true if the tile exists on FS and is going to try to open it,
	 *          false if the tile not exists
	 */
	public abstract boolean loadMapTileToMemCacheAsync(final Tile tile,
			final IHandler callback, final String cacheURL,
			final Cancellable cancellable, final TileDrawer drawer, final boolean prepend) throws IOException;

	/**
	 * Saves a tile to disk following a Quadtree strategy
	 * 
	 * @param someData
	 *            The array of bytes representing the tile
	 * @param tile
	 *            The tile
	 * @throws IOException
	 */
	public abstract void saveFile(final byte[] someData, Tile tile) throws IOException;

	public abstract void destroy();

	public abstract ITileFileSystemStrategy getStrategy();

	public abstract void setStrategy(ITileFileSystemStrategy strategy);

	public abstract TileCache getTileCache();

	public abstract Set getPendingQueue();

	/**
	 * @see TileProvider.MODE_*
	 */
	public abstract void setMode(int mode);
	
	public abstract int getMode();

}