package es.prodevelop.tilecache.provider.filesystem.impl;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.util.logging.Level;

import es.prodevelop.gvsig.mini.common.IBitmap;
import es.prodevelop.gvsig.mini.common.IContext;
import es.prodevelop.gvsig.mini.common.IHandler;
import es.prodevelop.gvsig.mini.common.impl.Tile;
import es.prodevelop.gvsig.mini.map.GeoUtils;
import es.prodevelop.gvsig.mini.utiles.Cancellable;
import es.prodevelop.gvsig.mini.utiles.Constants;
import es.prodevelop.gvsig.mini.utiles.WorkQueue;
import es.prodevelop.tilecache.draw.TileDrawer;
import es.prodevelop.tilecache.provider.TileCache;
import es.prodevelop.tilecache.provider.TileProvider;
import es.prodevelop.tilecache.provider.filesystem.strategy.ITileFileSystemStrategy;

/**
 * FileSystemProvider that makes all the operations to load a tile in
 * background. It has to notify to a Downloader when a tile has been retrieved
 * from the file system or not to allow the Downloader run or not.
 * 
 * @author aromeu
 * 
 */
public class BackgroundFileSystemProvider extends TileFilesystemProvider {

	private TileProvider tileProvider;

	public BackgroundFileSystemProvider(int maxFSCacheByteSize,
			TileCache cache, IContext context,
			ITileFileSystemStrategy strategy, int mode, IBitmap offlineBitmap,
			TileProvider tileProvider) {
		super(maxFSCacheByteSize, cache, context, strategy, mode, offlineBitmap);
		this.tileProvider = tileProvider;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seees.prodevelop.tilecache.provider.filesystem.ITileFileSystemProvider#
	 * loadMapTileToMemCacheAsync(java.lang.String,
	 * es.prodevelop.gvsig.mini.common.IHandler, int[], java.lang.String, int,
	 * java.lang.String)
	 */
	public synchronized boolean loadMapTileToMemCacheAsync(final Tile tile,
			final IHandler callback, final String cacheURL,
			final Cancellable cancellable, final TileDrawer drawer,
			final boolean prepend) throws IOException {

		// long t1 = System.currentTimeMillis();
		if (this.mPending.contains(cacheURL))
			return true;

		if (mode == TileProvider.MODE_CONNECTED)
			return false;

		mPending.add(cacheURL);

		es.prodevelop.gvsig.mini.utiles.WorkQueue.getInstance().execute(
				new Runnable() {
					// @Override
					public void run() {
						OutputStream out = null;
						ByteArrayOutputStream dataStream = null;

						boolean succeed = false;
						int retryCount = 0;

						// final String formattedTileURLString =
						// TileCache.format(aTileURLString);
						final String quadDir = strategy
								.getRelativeToLayerDirTilePath(tile.tile,
										tile.zoomLevel);

						String st = getSDDir();
						final File f = new File(new StringBuffer().append(st)
								.append(File.separator).append(
										Constants.APP_DIR).append(
										File.separator).append(
										Constants.MAPS_DIR).append(
										File.separator).append(tile.layerName)
								.append(strategy.getLayerNameSuffix()).append(
										File.separator).append(quadDir).append(
										strategy.getTileNameSuffix())
								.toString());

						if (!f.exists()) {
							if (mode == TileProvider.MODE_OFFLINE
									|| mode == TileProvider.MODE_DELETE) {
								// context.sendMessage(callback, aTileURLString,
								// MAPTILEFSLOADER_NOTFOUND_ID);
								context.sendMessage(callback, instantiateEvent(
										tile, ""), MAPTILEFSLOADER_SUCCESS_ID);
								mPending.remove(cacheURL);
								// long t2 = System.currentTimeMillis();
								//System.out.println("time to check if exists: "
								// + (t2 - t1)
								// + ".ms");
								// FIXME: true or false?
								// remove from downloader pending
//								System.out.println("0");
								return;
							} else {
								// long t2 = System.currentTimeMillis();
								//System.out.println("time to check if exists: "
								// + (t2 - t1)
								// + ".ms");
								// download with downloader
//								mPending.remove(cacheURL);
								tileProvider.downloadFromServer(tile, cacheURL,
										cancellable, drawer, prepend);
//								System.out.println("1");
								return;
							}
						} else {
							if (f.length() <= 0
									|| mode == TileProvider.MODE_DELETE) {
								f.delete();
//								mPending.remove(cacheURL);
								if (mode == TileProvider.MODE_DELETE) {
									context
											.sendMessage(callback,
													instantiateEvent(tile, f
															.getPath()),
													MAPTILEFSLOADER_DELETED_ID);
								} else {
									tileProvider.downloadFromServer(tile, cacheURL,
											cancellable, drawer, prepend);
								}
								// long t2 = System.currentTimeMillis();
								// System.out.println("void file: " + (t2 - t1)
								// + ".ms");
								// download with downloader																
//								System.out.println("2");
								return;
							} else if (mode == TileProvider.MODE_UPDATE) {
								// long t2 = System.currentTimeMillis();
								// System.out.println("mode update: " + (t2 -
								// t1) + ".ms");
								// download with downloader
//								mPending.remove(cacheURL);
								tileProvider.downloadFromServer(tile, cacheURL,
										cancellable, drawer, prepend);
//								System.out.println("3");
								return;
							}
						}

						FileInputStream in;
						try {
							in = new FileInputStream(f);

							while (retryCount <= TileProvider.NUM_RETRY
									&& !succeed) {
								try {
									dataStream = new ByteArrayOutputStream();

									out = new BufferedOutputStream(dataStream,
											Constants.IO_BUFFER_SIZE);
									Constants.copy(in, out);
									out.flush();

									final byte[] data = dataStream
											.toByteArray();
									if (data.length <= 0) {
										throw new IOException("");
									}

									if (cancellable.getCanceled()) {
										mPending.clear();
										WorkQueue.getInstance()
												.clearPendingTasks();
										context.sendMessage(callback,
												instantiateEvent(tile, f
														.getPath()),
												GeoUtils.TILEPROVIDER_CANCELED);
										return;
									}

									final IBitmap bmp = context
											.decodeByteArray(data, 0,
													data.length);

									if (mCache != null)
										mCache.putTile(cacheURL, bmp);

									context
											.sendMessage(callback,
													instantiateEvent(tile, f
															.getPath()),
													MAPTILEFSLOADER_SUCCESS_ID);
									succeed = true;

									if (drawer != null)
										drawer.drawTile(tile.clone(), bmp);

									// log.log(Level.FINE,"Loaded from disk: " +
									// aTileURLString);
								} catch (java.lang.OutOfMemoryError ex) {
									System.gc();
									// log.log(Level.SEVERE,
									// "OutOfMemoryError: ",
									// ex);
									mCache.onLowMemory();
									if (retryCount == TileProvider.NUM_RETRY)
										context.sendMessage(callback,
												instantiateEvent(tile, f
														.getPath()),
												MAPTILEFSLOADER_OOM_ID);
								} catch (Exception e) {
									if (retryCount == TileProvider.NUM_RETRY)
										context.sendMessage(callback,
												instantiateEvent(tile, f
														.getPath()),
												MAPTILEFSLOADER_FAIL_ID);
								} finally {
									Constants.closeStream(in);
									Constants.closeStream(dataStream);
									Constants.closeStream(out);
									retryCount++;
								}
							}
						} catch (FileNotFoundException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
						if (mode == TileProvider.MODE_OFFLINE && !succeed) {
							mCache.putTile(cacheURL, offlineBitmap);
							context.sendMessage(callback,

							instantiateEvent(tile, f.getPath()),
									MAPTILEFSLOADER_SUCCESS_ID);
//							mPending.remove(cacheURL);
						}
						mPending.remove(cacheURL);
					}
				}, prepend);
		return true;
	}
	
	public synchronized void saveFile(final byte[] someData, final Tile tile) throws IOException {		
		super.saveFile(someData, tile);
		this.mPending.remove(tile.getTileString());
	}

}
