/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2009 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2009.
 *   author Alberto Romeu aromeu@prodevelop.es 
 *   author Ruben Blanco rblanco@prodevelop.es 
 *   
 */

package es.prodevelop.tilecache.renderer;

import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.Pixel;
import es.prodevelop.gvsig.mini.geom.Point;
import es.prodevelop.gvsig.mini.projection.TileConversor;
import es.prodevelop.gvsig.mobile.fmap.proj.Projection;
import es.prodevelop.tilecache.util.Tags;

/**
 * This MapRenderer is intended for servers where Y parameter of tiles has the
 * origin at Equator and takes negative values to north and positive to south.
 * 
 * @author aromeu
 * @author rblanco
 * 
 */
public class EquatorRenderer extends OSMMercatorRenderer {

	

	String firstPart = "";
	String xParm = "";
	String yParm = "";
	String zParm = "";

	private EquatorRenderer(final String[] aBaseUrl, final String aName,
			final String aImageFilenameEnding, final int aZoomMax, final int aZoomMin,
			final int aTileSizePX, final int type,
			final Point origin, final Extent maxExtent, final String SRS, final double[] resolutions, Extent offExtent) {
		super(aBaseUrl, aName, aImageFilenameEnding, aZoomMax, aZoomMin, aTileSizePX,
				type, origin, maxExtent, SRS, resolutions, offExtent);
	}

	/**
	 * Instantiates a new EquatorRenderer
	 * 
	 * @param aBaseUrl
	 *            The base URL with the parameters separated by '#' for example:
	 *            http://theurlofthesever.com/something#&x=##&y=##&z=#
	 * @param aName
	 *            The name of the layer
	 * @param aImageFilenameEnding
	 *            The format of the tiles (png, jpg, jpeg, etc.)
	 * @param aZoomMax
	 *            The max zoom level of the layer
	 * @param aTileSizePX
	 *            The tile size in pixels
	 * @param type
	 *            MapRenderer.EQUATOR_RENDERER
	 * @param origin
	 *            The origin of the maxExtent, can be de left bottom corner, or
	 *            the left upper corner of the extent
	 * @param maxExtent
	 *            The max boundary box of the layer
	 * @param SRS
	 *            The SRS code
	 * @param resolutions
	 *            pixel resolution of each zoom level
	 */
	public static EquatorRenderer getEquatorRenderer(final String[] aBaseUrl,
			final String aName, final String aImageFilenameEnding,
			final int aZoomMax, final int aZoomMin, final int aTileSizePX, final int type,
			final Point origin, final Extent maxExtent, final String SRS, final double[] resolutions, Extent offExtent) {

		EquatorRenderer r = new EquatorRenderer(aBaseUrl, aName,
				aImageFilenameEnding, aZoomMax, aZoomMin, aTileSizePX, type, origin, maxExtent, SRS, resolutions, offExtent);
		return r;
	}

	@Override
	public String getTileURLString(final int[] tileID, final int zoomLevel) {
		try {
			int ytile = (((1 << (zoomLevel)) >> 1) - 1 - tileID[0]);
			String baseURL = this.getBASEURL();

			if (firstPart.compareTo("") == 0 || xParm.compareTo("") == 0
					|| yParm.compareTo("") == 0 || zParm.compareTo("") == 0) {
				firstPart = baseURL.substring(0,
						baseURL.indexOf("#"));
				String secondPart = baseURL.substring(
						baseURL.indexOf("#") + 1,
						baseURL.length());
				xParm = secondPart.substring(0, secondPart.indexOf("#"));
				secondPart = secondPart.substring(secondPart.indexOf("#") + 1,
						secondPart.length());
				yParm = secondPart.substring(secondPart.indexOf("#") + 1,
						secondPart.indexOf("#", 1));
				secondPart = secondPart.substring(
						secondPart.indexOf("#", 1) + 1, secondPart.length());
				zParm = secondPart.substring(secondPart.indexOf("#") + 1,
						secondPart.indexOf("#", 1));
			}
			
			firstPart = baseURL.substring(0,
					baseURL.indexOf("#"));

			return new StringBuilder().append(firstPart).append(xParm).append(
					tileID[1]).append(yParm).append(ytile).append(zParm)
					.append(17 - zoomLevel + 1).toString();
		} catch (Exception e) {
			
			return null;
		}
	}	

	public Extent getExtentIncludingCenter(Point center, int zoomLevel,
			Point origin) {
		Extent ex = null;
		Pixel tileNumber = null;
		String profile = null;

		try {

			Point p = TileConversor.mercatorToLatLon(center.getX(), center
					.getY());

			tileNumber = TileConversor.getTileNumber(p.getX(), p.getY(),
					zoomLevel);
			ex = TileConversor.tileOSMMercatorBounds(tileNumber.getX(),
					tileNumber.getY(), zoomLevel);

		} catch (final Exception e) {
			e.printStackTrace();
		}
		return ex;
	}

	public String buildQuery(final Extent e, final int zoomLevel,
			final double resolution) {
		final Point center = e.getCenter();
		int zoomYahoo = Tags.YAHOO_MIN_ZOOM_LEVEL - zoomLevel + 1;

		Pixel tileN = this.getTileNumber(e, zoomLevel, resolution, null);

		StringBuffer query = new StringBuffer().append(getURL()).append("&x=")
				.append(tileN.getX()).append("&y=").append(tileN.getY())
				.append("&z=").append(zoomYahoo).append(".jpg");

		return query.toString();
	}

	/**
	 * Calculates the quadkey string of the current tile
	 * 
	 * @param zoomLevel
	 *            The current zoomLevel of the map
	 * @return A StringBuffer with the quadkey String
	 */
	public StringBuffer getQuadkey(final int tileX, final int tileY,
			final int zoomLevel) {
		try {
			if (tileX == -1) {
				return null;
			} else {
				return TileConversor.tileXYToQuadKey(tileX, tileY, zoomLevel);
			}
		} catch (final Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public Pixel getTileNumber(final Extent e, final int zoomLevel,
			final double resolution, final Projection proj) {
		Point p = TileConversor.mercatorToLatLon(e.getCenter().getX(), e
				.getCenter().getY());
		return TileConversor.getYahooTileNumber(p.getX(), p.getY(), zoomLevel);
	}

	// @Override
	// public void setZoomLevel(int zoomLevel) {
	// if (zoomLevel == 0)
	// zoomLevel = 1;
	// super.setZoomLevel(zoomLevel);
	// }
}
