/* TileRasterCache, a library to download tiles from several sources.
 *
 * Copyright (C) 2010 Prodevelop SL.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *  
 *   Prodevelop Integracin de Tecnologas SL
 *   Plaza don Juan de Vilarrasa , 14-5
 *   46000 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *   
 *   author: Alberto Romeu Carrasco (aromeu@prodevelop.es)
 */

package es.prodevelop.tilecache.renderer;

/**
 * A singleton to register and retrieve MapRendererFactory
 * @author aromeu
 *
 */
public class MapRendererManager {
	
	private IMapRendererFactory mapRendererFactory;
	private static MapRendererManager manager;
	
	/**
	 * Singleton access to MapRendererManager
	 * @return The MapRenderermanager
	 */
	public static MapRendererManager getInstance() {
		if (manager == null) {
			manager = new MapRendererManager();
		}
		return manager;
	}
	
	/**
	 * Registers a IMapRendererFactory
	 * @param renderer The ImapRendererFactory
	 * @throws Exception if the param is null
	 */
	public void registerMapRendererFactory(IMapRendererFactory renderer) throws Exception {
		if (renderer == null)
			throw new Exception("Null MapRendererFactory");
		
		this.mapRendererFactory = renderer;
	}
	
	/**
	 * Retrieves the previously registered IMapRendererFactory
	 * @return The IMapRendererFactory registered
	 * @throws Exception if no IMapRendererFactory was previously registered
	 */
	public IMapRendererFactory getMapRendererFactory() throws Exception {
		if (this.mapRendererFactory == null) {
			throw new Exception("Null MapRendererFactory");
		}
		return this.mapRendererFactory;
	}
	

}
