package es.prodevelop.tilecache.summary;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.Iterator;

import es.prodevelop.gvsig.mini.common.CompatManager;
import es.prodevelop.gvsig.mini.utiles.Constants;
import es.prodevelop.tilecache.renderer.MapRenderer;

/**
 * Writes a file in the same folder of a tileset cached with a summary of the
 * tiles downloaded, the size in bytes, the date, the MapRenderer, etc.
 * 
 * @author aromeu
 * 
 */
public class SummaryWriter {

	private SummaryInfo info;

	public SummaryWriter(SummaryInfo info) {
		this.info = info;
	}

	public void write() {
		BufferedWriter out = null;
		FileWriter logwriter = null;
		try {
			String prop = "";

			String st = CompatManager.getInstance().getRegisteredContext()
					.getExternalStorageDirectoryPath();

			// Open or create the file where to store the context
			// String dirPath = st + File.separator + Constants.APP_DIR
			// + File.separator + Constants.MAPS_DIR + File.separator
			// + info.getRenderer().getNAME()+info.getLayerNameSuffix() +
			// File.separator;
			File f = new File(st + File.separator + info.getVersion() + "_"
					+ SummaryInfo.FILE_NAME);

			if (!f.exists()) {
				File dirFile = new File(st);
				dirFile.mkdirs();
				f.createNewFile();
			} else { // If exists, let's delete it and create again
				f.delete();
				f.createNewFile();
			}

			logwriter = new FileWriter(f, false);
			out = new BufferedWriter(logwriter);

			// Let's persist all the properties of the hashmap

			// we set the extent of the renderer to the whole one prior to
			// persist
			info.getRenderer().setExtent(info.getRendererExtent());
			info.getRenderer().setNAME(
					info.getRenderer().getNAME() + MapRenderer.NAME_SEPARATOR
							+ info.getCityName());
			out.write(info.getRenderer().toString());
			// FIXME: restore the extent of the renderer to the
			// info.downloadedExtent ????
			out.write("\n");
			Iterator iter = info.getProperties().keySet().iterator();
			while (iter.hasNext()) {
				prop = (String) iter.next();
				out.write(prop);
				out.write("=");
				out.write(info.getValue(prop).toString());
				out.write("\n");
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				out.close();
			} catch (Exception ignore) {

			}

			try {
				logwriter.close();
			} catch (Exception ignore) {

			}
		}
	}

}
